@@ -1,6 +1,6 @@
 'BIOMOD_EnsembleModeling' <- function( modeling.output,
                                        chosen.models = 'all',
-                                       em.by = 'all',
+                                       em.by = 'PA_dataset+repet',
                                        eval.metric = 'all',
                                        eval.metric.quality.threshold = NULL,
                                        models.eval.meth = c('KAPPA','TSS','ROC'),
@@ -89,20 +89,31 @@
     }
     
     ##### !!!!!! TO DO -> select appropriate part of dataset according to em.by
+    obs <-  get_formal_data(modeling.output,'resp.var')
+    expl <- get_formal_data(modeling.output,'expl.var')
+    
+    ## subselection of observations according to dataset used to produce ensemble models
     if(em.by %in% c("PA_dataset",'PA_dataset+algo','PA_dataset+repet')){
-      obs <-  get_formal_data(modeling.output,'resp.var')
-      expl <- get_formal_data(modeling.output,'expl.var')
-      if(head(unlist(strsplit(assemb,"_")),1) != 'AllData'){
-        kept_cells <- get_formal_data(modeling.output)@PA[, paste(head(unlist(strsplit(assemb,"_")),1))]
+      if(unlist(strsplit(assemb,"_"))[3] != 'AllData'){
+        if(class(get_formal_data(modeling.output)) == "BIOMOD.formated.data.PA"){
+          kept_cells <- get_formal_data(modeling.output)@PA[, unlist(strsplit(assemb,"_"))[3]]
+        } else {
+          kept_cells <- rep(T, length(obs))
+        }
+        
         obs <- obs[kept_cells]
         expl <- expl[kept_cells, ,drop=F]
-      }                              
-    }   
-    if(em.by %in% c("algo","all") ){
-      ## we need to take all data even if it is much better to have 
-      obs <-  get_formal_data(modeling.output,'resp.var')
-      expl <- get_formal_data(modeling.output,'expl.var')
-    }                                        
+      }
+    } 
+    
+    ## subselection of observations according to dataset used to produce ensemble models is done at evaluation step
+    
+    
+#     if(em.by %in% c("algo","all") ){
+#       ## we need to take all data even if it is much better to have 
+#       obs <-  get_formal_data(modeling.output,'resp.var')
+#       expl <- get_formal_data(modeling.output,'expl.var')
+#     }                                        
     # remove na
     obs[is.na(obs)] <- 0
     
@@ -118,7 +129,7 @@
     for(eval.m in eval.metric){
       
       # define model name
-      base_model_name <- paste(modeling.output@sp.name,"_",assemb,"_",eval.m,'_' ,sep="")
+#       base_model_name <- paste(modeling.output@sp.name,"_",assemb,"_",eval.m,'_' ,sep="")
       models.kept <- needed_predictions$models.kept[[eval.m]]
       models.kept.scores <- needed_predictions$models.kept.scores[[eval.m]]
       
@@ -131,7 +142,8 @@
         if(algo == 'prob.mean'){
           cat("\n   > Mean of probabilities...")
           
-          model_name <- paste(base_model_name,"EMmean",sep="")
+#           model_name <- paste(base_model_name,"EMmean",sep="")
+          model_name <- paste(modeling.output@sp.name,"_","EMmeanBy",eval.m, "_", assemb ,sep="")
           
           model.bm <- new("EMmean_biomod2_model",
                            model = models.kept,
@@ -148,7 +160,8 @@
         # 2. CV of probabilities -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- #
         if(algo == 'prob.cv'){
           cat("\n   > Coef of variation of probabilities...")
-          model_name <- paste(base_model_name,"EMcv",sep="")
+#           model_name <- paste(base_model_name,"EMcv",sep="")
+          model_name <-paste(modeling.output@sp.name,"_","EMcvBy",eval.m, "_", assemb ,sep="")
           
           model.bm <- new("EMcv_biomod2_model",
                            model = models.kept,
@@ -165,7 +178,8 @@
         # 3. Median of probabilities -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- #
         if(algo == 'prob.median'){
           cat("\n   > Median of ptobabilities...")
-          model_name <- paste(base_model_name,"EMmedian",sep="")
+#           model_name <- paste(base_model_name,"EMmedian",sep="")
+          model_name <- paste(modeling.output@sp.name,"_","EMmedianBy",eval.m, "_", assemb ,sep="")
           
           model.bm <- new("EMmedian_biomod2_model",
                            model = models.kept,
@@ -184,7 +198,8 @@
           cat("\n   > Confidence Interval...")
           
           ## Quantile inferior
-          model_name <- paste(base_model_name,"EMciInf",sep="")
+#           model_name <- paste(base_model_name,"EMciInf",sep="")
+          model_name <- paste(modeling.output@sp.name,"_","EMciInfBy",eval.m, "_", assemb ,sep="")
           
           model.bm <- new("EMci_biomod2_model",
                            model = models.kept,
@@ -202,7 +217,8 @@
 
         if(algo == 'prob.ci.sup'){
           ## Quantile superior
-          model_name <- paste(base_model_name,"EMciSup",sep="")
+#           model_name <- paste(base_model_name,"EMciSup",sep="")
+          model_name <- paste(modeling.output@sp.name,"_","EMciSupBy",eval.m, "_", assemb ,sep="")          
           
           model.bm <- new("EMci_biomod2_model",
                            model = models.kept,
@@ -222,7 +238,8 @@
         # 5. Comitee averaging of probabilities -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= #
         if(algo == 'committee.averaging'){      
           cat("\n   >  Comittee averaging...")
-          model_name <- paste(base_model_name,"EMca",sep="")
+#           model_name <- paste(base_model_name,"EMca",sep="")
+          model_name <- paste(modeling.output@sp.name,"_","EMcaBy",eval.m, "_", assemb ,sep="")
           
           models.kept.tresh <- unlist(lapply(models.kept, function(x){
             mod <- tail(unlist(strsplit(x,"_")), 3)[3]
@@ -252,8 +269,9 @@
         # 6. weighted mean of probabilities -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= #
         if(algo == 'prob.mean.weight'){
           cat("\n   > Prababilities wegthing mean...")
-          model_name <- paste(base_model_name,"EMwmean",sep="")
-          
+#           model_name <- paste(base_model_name,"EMwmean",sep="")
+          model_name <- paste(modeling.output@sp.name,"_","EMwmeanBy",eval.m, "_", assemb ,sep="")          
+
           # remove SRE models if ROC
           models.kept.tmp <- models.kept
           models.kept.scores.tmp <- models.kept.scores
@@ -315,7 +333,7 @@
         }
         
         #### Models Evaluation ####
-        pred.bm <- predict(model.bm, expl, formal_predictions=needed_predictions$predictions[,model.bm@model, drop=F] )
+        pred.bm <- predict(model.bm, expl, formal_predictions=needed_predictions$predictions[,model.bm@model, drop=F], on_0_1000 = T )
         
         if(exists('eval.obs') & exists('eval.expl')){
           eval_pred.bm <- predict(model.bm, eval.expl)
@@ -325,28 +343,59 @@
         # Model evaluation stuff =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= #
         if( length(models.eval.meth) ){
           cat("\n\t\t\tEvaluating Model stuff...")
- 
-          cross.validation <- sapply(models.eval.meth,
-                                     Find.Optim.Stat,
-                                     Fit = pred.bm,
-                                     Obs = obs)
           
-          rownames(cross.validation) <- c("Testing.data","Cutoff","Sensitivity", "Specificity")
+          if(algo == 'prob.cv'){ ## switch of evalutaion process
+            cross.validation <- matrix(NA,4,length(models.eval.meth), 
+                                       dimnames = list(c("Testing.data","Cutoff","Sensitivity", "Specificity"),
+                                                       models.eval.meth))
+          } else {
+            if(em.by == "PA_dataset+repet"){ ## select the same evaluation data than formal models
+              ## get formal models calib/eval lines
+              calib_lines <- get_calib_lines(modeling.output)
+              ## get info on wich dataset and which repet this ensemble model is based on
+              pa_dataset_id <- paste("_", unlist(strsplit(assemb,"_"))[3], sep="")
+              repet_id <- paste("_", unlist(strsplit(assemb,"_"))[2], sep="")
+              ## define and extract the subset of points model will be evaluated on
+              if (repet_id == "_Full"){
+                eval_lines <- rep(T, length(pred.bm))
+              } else {
+                eval_lines <- ! na.omit(calib_lines[ , repet_id, pa_dataset_id])
+              }
+            } else {
+              eval_lines <- rep(T, length(pred.bm))
+            }
+            
+            cross.validation <- sapply(models.eval.meth,
+                                       Find.Optim.Stat,
+                                       Fit = pred.bm[eval_lines],
+                                       Obs = obs[eval_lines])
+            rownames(cross.validation) <- c("Testing.data","Cutoff","Sensitivity", "Specificity")
+          }
+ 
+          
           
           if(exists('eval_pred.bm')){
             
-            true.evaluation <- sapply(models.eval.meth,
-                                      function(x){
-                                        return( Find.Optim.Stat(Stat = x,
-                                                                Fit = eval_pred.bm,
-                                                                Obs = eval.obs,
-                                                                Fixed.thresh = cross.validation["Cutoff",x]) )
-                                      })
+            if(algo == 'prob.cv'){ ## switch of evalutaion process
+              cross.validation <- matrix(NA,5,length(models.eval.meth), 
+                                         dimnames = list(c("Testing.data","Evaluating.data","Cutoff","Sensitivity", "Specificity"),
+                                                         models.eval.meth))
+            } else {
+              true.evaluation <- sapply(models.eval.meth,
+                                        function(x){
+                                          return( Find.Optim.Stat(Stat = x,
+                                                                  Fit = eval_pred.bm,
+                                                                  Obs = eval.obs,
+                                                                  Fixed.thresh = cross.validation["Cutoff",x]) )
+                                        })
+              
+              
+              cross.validation <- rbind(cross.validation["Testing.data",], true.evaluation)
+              rownames(cross.validation) <- c("Testing.data","Evaluating.data","Cutoff","Sensitivity", "Specificity")
+            }
             
             
-            cross.validation <- rbind(cross.validation["Testing.data",], true.evaluation)
             
-            rownames(cross.validation) <- c("Testing.data","Evaluating.data","Cutoff","Sensitivity", "Specificity")
           }
           
           ## store results
@@ -551,20 +600,23 @@
   
   if(em.by == 'PA_dataset'){
     for(dat in .extractModelNamesInfo(chosen.models, info='data.set')){
-      assembl.list[[paste(dat,"_AllRun", sep="")]] <- chosen.models[grep(paste("_",dat,"_",sep=""), chosen.models)]
+#       assembl.list[[paste(dat,"_AllRun", sep="")]] <- chosen.models[grep(paste("_",dat,"_",sep=""), chosen.models)]
+      assembl.list[[paste("mergedAlgo_mergedRun_", dat, sep="")]] <- chosen.models[grep(paste("_",dat,"_",sep=""), chosen.models)]
     }
     return(assembl.list)
   }
   
   if(em.by == 'algo'){
     for(algo in .extractModelNamesInfo(chosen.models, info='models')){
-      assembl.list[[paste(algo,"_AllRun", sep="")]] <- chosen.models[grep(paste("_",algo,sep=""), chosen.models)]
+#       assembl.list[[paste(algo,"_AllRun", sep="")]] <- chosen.models[grep(paste("_",algo,sep=""), chosen.models)]
+      assembl.list[[paste(algo,"_mergedRun_mergedData", sep="")]] <- chosen.models[grep(paste("_",algo,sep=""), chosen.models)]
     }
     return(assembl.list)
   }
   
   if(em.by == 'all'){
-    assembl.list[["TotalConsensus"]] <- chosen.models
+#     assembl.list[["TotalConsensus"]] <- chosen.models
+    assembl.list[[paste("mergedAlgo_mergedRun_mergedData", sep="")]] <- chosen.models
     return(assembl.list)
   }
   
@@ -574,7 +626,8 @@
         mod.tmp <- intersect(x=grep(paste("_",dat,"_",sep=""), chosen.models), 
                              y=grep(paste("_",repet,"_",sep=""), chosen.models))
         if(length(mod.tmp)){
-          assembl.list[[paste(dat,"_",repet,'_AllAlgos', sep="")]] <- chosen.models[mod.tmp]
+#           assembl.list[[paste(dat,"_",repet,'_AllAlgos', sep="")]] <- chosen.models[mod.tmp]
+          assembl.list[[paste("mergedAlgo_",repet,"_",dat, sep="")]] <- chosen.models[mod.tmp]
         }
       }
     }
@@ -587,7 +640,8 @@
         mod.tmp <- intersect(x=grep(paste("_",dat,"_",sep=""), chosen.models), 
                              y=grep(paste("_",algo,sep=""), chosen.models))
         if(length(mod.tmp)){
-          assembl.list[[paste(dat,"_AllRepet_",algo, sep="")]] <- chosen.models[mod.tmp]
+#           assembl.list[[paste(dat,"_AllRepet_",algo, sep="")]] <- chosen.models[mod.tmp]
+          assembl.list[[paste(algo,"_mergedRun_", dat, sep="")]] <- chosen.models[mod.tmp]
         }
       }
     }
